// A basic OpenGL program displaying OpenGL geometric primitives
// Burkhard Wuensche, 27 Feb 2002

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include <stdio.h>
#include <string.h>

const int windowWidth=650;
const int windowHeight=450;

// example 1: Use the given vertices to draw points, lines, a line strip, a line loop a triangle fan and a polygon.
const int numVertices1=8;
const float vertices1[numVertices1][2] = {{10.0,10.0},{30.0,0.0},{50.0,0.0},{70.0,20.0},
										{70.0,40.0},{50.0,80.0},{30.0,80.0},{10.0,50.0}};
const int numModes1=6;
const GLenum modes1[numModes1]={GL_POINTS, GL_LINES, GL_LINE_STRIP, GL_LINE_LOOP, GL_TRIANGLE_FAN, GL_POLYGON};
const char names1[numModes1][20]={"GL_POINTS", "GL_LINES", "GL_LINE_STRIP", "GL_LINE_LOOP", "GL_TRIANGLE_FAN", "GL_POLYGON"};

// example 2: Use the given vertices to draw points, triangles, a triangle strip and a quad strip
const int numVertices2=6;
const float vertices2[numVertices2][2] =  {{10.0,10.0},{10.0,50.0},{30.0,0.0},{30.0,80.0},{50.0,20.0},{50.0,60.0}};
const int numModes2=4;
const GLenum modes2[numModes2]={GL_POINTS, GL_TRIANGLES, GL_TRIANGLE_STRIP, GL_QUAD_STRIP};
const char names2[numModes2][20]={"GL_POINTS", "GL_TRIANGLES", "GL_TRIANGLE_STRIP", "GL_QUAD_STRIP"};

//example3: Use the given vertices to draw points and quadrilaterals
const int numVertices3=8;
const float vertices3[numVertices3][2] = {{10.0,10.0},{10.0,50.0},{30.0,80.0},{30.0,0.0},
										{50.0,0.0},{50.0,80.0},{70.0,40.0},{70.0,20.0}};
const int numModes3=2;
const GLenum modes3[numModes3]={GL_POINTS, GL_QUADS};
const char names3[numModes3][20]={"GL_POINTS", "GL_QUADS"};

// Draws the string s at the current raster position using the given font.
// If no font argument is given GLUT_BITMAP_9_BY_15  is used as default
inline void drawString(const char* s, void* font=GLUT_BITMAP_9_BY_15){
	while(*s!='\0') glutBitmapCharacter(font, *(s++));
}

void display(void)
{
	// clear all pixels in frame buffer
	glClear(GL_COLOR_BUFFER_BIT);
	glPointSize(3.0f);
	int i,j,dy,dx=110;

	// =================
	// = First example =
	// =================
	// Draw the geometric objects
	glColor3f(1.0, 0.1, 0.2);	// reddish colour
	dy=340;						// draw objects at the top of the window
	glPushMatrix();
	glTranslatef(0.0, (GLfloat) dy, 0.0);
	for(j=0;j<numModes1;j++)
	{
		glBegin(modes1[j]);
		for(i=0;i<numVertices1;i++)
		{
			if (i%3==0) glColor3f(1.0,0.0,0.0);
			if (i%3==1) glColor3f(0.0,1.0,0.0);
			if (i%3==2) glColor3f(0.0,0.0,1.0);
			glVertex2fv(vertices1[i]);
		}
		glEnd();
		glTranslatef((GLfloat) dx, 0.0, 0.0);	// draw next object to right of the previous object
	}
	glPopMatrix();
	// Draw the vertex numbers next to the points
	glColor3f(0.0, 0.0, 0.0);	// black colour
	for(i=0;i<numVertices1;i++)
	{
		glRasterPos2f(vertices1[i][0]+3, dy+vertices1[i][1]);
		char* s=new char[2]; sprintf(s,"%d",i); drawString(s);
	}
	// Draw the name of each geometric primitive
	for(j=0;j<numModes1;j++)
	{
		glRasterPos2f(dx*j+5, dy-20);
		drawString(names1[j],GLUT_BITMAP_HELVETICA_10);
	}


	// ==================
	// = Second example =
	// ==================
	// Draw the geometric objects
	glColor3f(1.0, 0.1, 0.2);	// reddish colour
	dy=190;						// draw objects in the middle of the window
	glPushMatrix();
	glTranslatef(0.0, (GLfloat) dy, 0.0);
	for(j=0;j<numModes2;j++)
	{
		glBegin(modes2[j]);
		for(i=0;i<numVertices2;i++)
			glVertex2fv(vertices2[i]);
		glEnd();
		glTranslatef((GLfloat) dx, 0.0, 0.0);
	}
	glPopMatrix();
	// Draw the vertex numbers next to the points
	glColor3f(0.0, 0.0, 0.0);	// black colour
	for(i=0;i<numVertices2;i++)
	{
		glRasterPos2f(vertices2[i][0]+3, dy+vertices2[i][1]);
		char* s=new char[2]; sprintf(s,"%d",i); drawString(s);
	}
	// Draw the name of each geometric primitive
	for(j=0;j<numModes2;j++)
	{
		glRasterPos2f(dx*j+5, dy-20);
		drawString(names2[j],GLUT_BITMAP_HELVETICA_10);
	}

	// =================
	// = Third example =
	// =================
	// Draw the geometric objects
	glColor3f(1.0, 0.1, 0.2);	// reddish colour
	dy=40;						// draw objects at the bottom of the window
	glPushMatrix();
	glTranslatef(0.0, (GLfloat) dy, 0.0);
	for(j=0;j<numModes3;j++)
	{
		glBegin(modes3[j]);
		for(i=0;i<numVertices3;i++)
			glVertex2fv(vertices3[i]);
		glEnd();
		glTranslatef((GLfloat) dx, 0.0, 0.0);
	}
	glPopMatrix();
	// Draw the vertex numbers next to the points
	glColor3f(0.0, 0.0, 0.0);	// black colour
	for(i=0;i<numVertices3;i++)
	{
		glRasterPos2f(vertices3[i][0]+3, dy+vertices3[i][1]);
		char* s=new char[2]; sprintf(s,"%d",i); drawString(s);
	}
	// Draw the name of each geometric primitive
	for(j=0;j<numModes3;j++)
	{
		glRasterPos2f(dx*j+5, dy-20);
		drawString(names3[j],GLUT_BITMAP_HELVETICA_10);
	}

	// start processing buffered OpenGL routines 
	glFlush ();
}

void init(void) 
{
	// select clearing color (for glClear)
	glClearColor (1.0, 1.0, 1.0, 0.0);	// RGB-value for white

	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	GLdouble halfWidth=(GLdouble) windowWidth/2.0;
	GLdouble halfHeight=(GLdouble) windowHeight/2.0;
	gluOrtho2D(0,windowWidth,0,windowHeight);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

// create a single buffered colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Geometric Primitives");
	init ();						// initialise view
	glutDisplayFunc(display);		// draw scene
	glutMainLoop();
	return 0;
}
